
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "nautilus-extension-enum-types.h"
#include "nautilus-info-provider.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "nautilus-info-provider.h" */

GType
nautilus_operation_result_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (NAUTILUS_OPERATION_COMPLETE), "NAUTILUS_OPERATION_COMPLETE", "complete" },
        { C_ENUM (NAUTILUS_OPERATION_FAILED), "NAUTILUS_OPERATION_FAILED", "failed" },
        { C_ENUM (NAUTILUS_OPERATION_IN_PROGRESS), "NAUTILUS_OPERATION_IN_PROGRESS", "in-progress" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("NautilusOperationResult"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

